/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 2.
 *
 * The Initial Developer of the Original Code is
 * Aaron Andersen.
 * Portions created by the Initial Developer are Copyright (C) 2___
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Aaron Andersen <aaron@xulplanet.com>
 *                 Matt Kennedy
 *                 Manuel Reimer <Manuel.Reimer@gmx.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// Globals
var goPrefBar = null;

var RDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
var mDatasource = null;
var FormatVersion = null;

var NC = "http://www.xulplanet.com/rdf/prefbar#";

function Init(go) {
    goPrefBar = go;

    var tmpInternalDS = RDFService.GetDataSourceBlocking("chrome://prefbar/content/prefbar.rdf");
    FormatVersion = ReadFormatVersion(tmpInternalDS);

    CheckForDataFile();

    var fDS = prefbarGetProfileDir("prefbar.rdf");
    mDatasource = RDFService.GetDataSourceBlocking(fDS);

    if (!CanReadFormat(mDatasource)) alert(goPrefBar.GetString("rdf.properties", "cantreadformat"));

    if (FormatUpdateNeeded(mDatasource)) {
        alert(goPrefBar.GetString("rdf.properties", "formatupdateneeded"));
        PerformFormatUpdates(mDatasource);
        alert(goPrefBar.GetString("rdf.properties", "formatupdatefinished"));
    }
}

function AddmDatasource(element) {
    element.database.AddDataSource(mDatasource);
    element.builder.rebuild();
}

function CheckForDataFile() {
    // hack becasue I can't seem to get an nsIFile for the data file directly;
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
        .getService(Components.interfaces.nsIProperties);
    var profDir = dirService.get("ProfD", Components.interfaces.nsIFile);

    var target = profDir.clone();
    target.append("prefbar.rdf");

    // If the "prefbar.rdf" file doesn't exist then we simply copy it to the profile directory
    if(target.exists()) return;

    // Thanks to Matt Kennedy for this code!

    var ioservice = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);

    target.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);

    var channel = ioservice.newChannel("chrome://prefbar/content/prefbar.rdf", null, null);
    var istream = channel.open();
    var ostream = Components.classes["@mozilla.org/network/file-output-stream;1"]
        .createInstance(Components.interfaces.nsIFileOutputStream);
    var scriptstream = Components.classes["@mozilla.org/scriptableinputstream;1"]
        .createInstance(Components.interfaces.nsIScriptableInputStream);
    scriptstream.init(istream);

    var len = scriptstream.available();
    var buffer = scriptstream.read(len);

    ostream.init(target, 0x08 | 0x02, 0600, 0);
    ostream.write(buffer, len);

    ostream.close();
    istream.close();
    scriptstream.close();

    // As we now have the latest prefbar.rdf installed we have to update the prefbar.version property.
    var nsIPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefBranch = nsIPref.getBranch("");
    prefBranch.setIntPref("extensions.prefbar.version", goPrefBar.prefbarVersion);
}


function prefbarGetProfileDir(subdir) {
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components.interfaces.nsIProperties);
  var profDir = dirService.get("ProfD", Components.interfaces.nsIFile);
  //var service = Components.classes["@mozilla.org/network/io-service;1"]
  //                        .getService(Components.interfaces.nsIIOService);
  var fileHandler = Components.classes["@mozilla.org/network/protocol;1?name=file"].getService(Components.interfaces.nsIFileProtocolHandler);

  // Fix for 1.2
  //if("getURLSpecFromFile"in service)
  //  return service.getURLSpecFromFile(profDir) + subdir;

  //var fileHandler = service.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
  return fileHandler.getURLSpecFromFile(profDir) + subdir;
}

/* This doesn't work as the "Flush" has to be called from the same
   context where the changes have been done for any reason
function FlushmDatasource()
{
    var remoteControll = mDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    remoteControll.Flush();
    //mDatasource.Flush();
}
*/



//
// Format Version handling... We never know how often we improve the format
// used by PrefBar in future, so we need a working version handling to do the
// update if needed
//

function ReadFormatVersion(datasource) {
    var infoSection = RDFService.GetResource("urn:prefbar:info");
    var fversionAttrib = RDFService.GetResource("http://www.xulplanet.com/rdf/prefbar#formatversion");
    var fversionValue = datasource.GetTarget(infoSection, fversionAttrib, true);
    if (!fversionValue) return 0;
    return fversionValue.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
}

function CanReadFormat(datasource) {
    var toread_version = ReadFormatVersion(datasource);
    return (toread_version <= FormatVersion);
}

function FormatUpdateNeeded(datasource) {
    var toread_version = ReadFormatVersion(datasource);
    return (toread_version < FormatVersion);
}

function PerformFormatUpdates(datasource) {
    // If we get new Formats in Future it may be best to "go step by step"
    // from format to format until we are current
    if (ReadFormatVersion(datasource) == 0)
        FormatUpdate0to1(datasource);
    if (ReadFormatVersion(datasource) == 1)
        FormatUpdate1to2(datasource);

}

// The "format 0 to 1" conversion is just the removing of the 
// "<br>"-replacement for newlines I've once implemented. There is no real
// reason to keep this and if someone would ask me why I ever added this I even
// would have no answer for that ;-). This function will also add the new
// "info section" to all files passed through this.
function FormatUpdate0to1(datasource) {
    var multilined_prefs =
        {"HTTP://WWW.XULPLANET.COM/RDF/PREFBAR#ONCLICK":true,
         "HTTP://WWW.XULPLANET.COM/RDF/PREFBAR#SETFUNCTION":true,
         "HTTP://WWW.XULPLANET.COM/RDF/PREFBAR#GETFUNCTION":true};
    var RButtons = RDFService.GetResource("urn:prefbar:buttons");

    var updatecontainer = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    updatecontainer.Init(datasource, RButtons);

    var updateelements = updatecontainer.GetElements();
    while (updateelements.hasMoreElements())
    {
	var updateelement = updateelements.getNext();
        
        var updateproperties = datasource.ArcLabelsOut(updateelement);
	while(updateproperties.hasMoreElements()) {
	    var updateproperty = updateproperties.getNext();
            var updatePropertyString = updateproperty.QueryInterface(Components.interfaces.nsIRDFResource).Value.toUpperCase();

            if (multilined_prefs[updatePropertyString] == true) {
                var Nvalue = datasource.GetTarget(updateelement, updateproperty, true);
                var value = Nvalue.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
                value = value.split("<br>").join("\n");
                value = value.split("brbr>").join("br>");
                var Nnewvalue = RDFService.GetLiteral(value);

                datasource.Change(updateelement, updateproperty, Nvalue, Nnewvalue);
            }
        }
    }
    var infoSection = RDFService.GetResource("urn:prefbar:info");
    var fversionAttrib = RDFService.GetResource("http://www.xulplanet.com/rdf/prefbar#formatversion");
    var newVersion = RDFService.GetLiteral(1);
    datasource.Assert(infoSection, fversionAttrib, newVersion, true); 
    datasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
}

// The "format 1 to 2" conversion will move all stuff from urn:prefbar:buttons
// to the two sequences urn:prefbar:browserbuttons:enabled and
// urn:prefbar:browserbuttons:disabled
function FormatUpdate1to2(datasource) {
    var REnabled = RDFService.GetResource("urn:prefbar:browserbuttons:enabled");
    var RDisabled = RDFService.GetResource("urn:prefbar:browserbuttons:disabled");
    var RButtons = RDFService.GetResource("urn:prefbar:buttons");

    var RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService();
    RDFCU = RDFCU.QueryInterface(Components.interfaces.nsIRDFContainerUtils);

    var CEnabled = RDFCU.MakeSeq(datasource, REnabled);
    var CDisabled = RDFCU.MakeSeq(datasource, RDisabled);
    var CButtons = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    CButtons.Init(datasource, RButtons);

    var REnabledPref = RDFService.GetResource("http://www.xulplanet.com/rdf/prefbar#enabled");

    // Move all elements to the new sequences and kill the "enabled" pref
    var elements = CButtons.GetElements();
    while (elements.hasMoreElements()) {
        var Relement = elements.getNext();
        var LEnabled = datasource.GetTarget(Relement, REnabledPref, true);
        if (LEnabled) {
            var enabled = LEnabled.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
            if (enabled == "true")
                CEnabled.AppendElement(Relement);
            else
                CDisabled.AppendElement(Relement);

            datasource.Unassert(Relement, REnabledPref, LEnabled);
        }

        if (elements.hasMoreElements())
            CButtons.RemoveElement(Relement, false);
        else
            CButtons.RemoveElement(Relement, true);
    }

    // Kill the old sequence
    var properties = datasource.ArcLabelsOut(RButtons);
    while(properties.hasMoreElements()) {
        var Rproperty = properties.getNext();
        var Lvalue = datasource.GetTarget(RButtons, Rproperty, true);
        datasource.Unassert(RButtons, Rproperty, Lvalue);
    }

    // Update FormatVersion
    var RinfoSection = RDFService.GetResource("urn:prefbar:info");
    var RversionAttrib = RDFService.GetResource("http://www.xulplanet.com/rdf/prefbar#formatversion");
    var newVersion = RDFService.GetLiteral(2);
    var oldVersion = datasource.GetTarget(RinfoSection, RversionAttrib, true);
    datasource.Change(RinfoSection, RversionAttrib, oldVersion, newVersion);

    // Save database
    datasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
}
